@@ -7,29 +7,22 @@ Sofar Ocean Technologies
 Authors: Pieter Bart Smit
 ======================
 
-Routines that can be used to save and load Spectral data.
+Routines that can be used to save and load data.
 
 Functions:
 
-- `load_spectrum`, load spectral data
-- `save_spectrum`, save spectral data
-- `_deserialize`, deserialize data
-- `_serialize`, serialize data
 
 
 How To Use This Module
 ======================
 (See the individual functions for details.)
 
-1. No need for direct import, use roguewave ``import roguewave`` or ``from roguewave import ...``.
-2. Save data: save_spectrum(data, filename)
-3. load data: data = load_spectrum(filename)
+
 
 """
 from roguewave.wavespectra.spectrum1D import WaveSpectrum1D
 from roguewave.wavespectra.spectrum2D import WaveSpectrum2D
 from roguewave.metoceandata import WaveBulkData, WindData, SSTData,BarometricPressure
-from roguewave import spectrum1D, spectrum2D
 from pandas import DataFrame, read_json
 from typing import Union, Dict, List
 from datetime import datetime
@@ -119,9 +112,9 @@ def object_hook(dictionary:dict):
                 df.index = [ x.tz_localize('utc') for x in df.index]
             return df
         elif dictionary['__class__'] == 'WaveSpectrum1D':
-            return spectrum1D(**dictionary['data'])
+            return WaveSpectrum1D(**dictionary['data'])
         elif dictionary['__class__'] == 'WaveSpectrum2D':
-            return spectrum2D(**dictionary['data'])
+            return WaveSpectrum2D(**dictionary['data'])
         elif dictionary['__class__'] == 'ndarray':
             return _b64_decode_numpy(dictionary['data'])
         elif dictionary['__class__'] == 'datetime':
@@ -143,13 +136,13 @@ def object_hook(dictionary:dict):
                 return dictionary
             # if directions -> create a wavespectrum2d. The keys directly map
             # to the constructor arguments.
-            return spectrum2D(**dictionary)
+            return WaveSpectrum2D(**dictionary)
         elif 'frequency' in dictionary:
             if isinstance(dictionary['frequency'],numpy.ndarray):
                 return dictionary
             # if not directions but has frequency -> create a wavespectrum1d.
             # The keys directly map to the constructor arguments.
-            return spectrum1D(**dictionary)
+            return WaveSpectrum1D(**dictionary)
         elif 'dataframe' in dictionary:
             df = read_json(dictionary['dataframe'])
             df['timestamp'] = df['timestamp'].apply(lambda x: x.tz_localize('utc'))
@@ -164,34 +157,7 @@ def load(filename: str) -> _UNION:
 
     :param filename: path to file to load.
     :return:
-        - WaveSpectrum1D
-          A single 1D spectrum
-
-        - WaveSpectrum2D
-          A single 2D spectrum
-
-        - List[WaveSpectrum1D]
-          List of 1D spectra
-
-        - List[WaveSpectrum2D],
-          List of 2D spectra.
-
-        - Dict[str, List[WaveSpectrum1D]]
-          Dictionary with Spotter/Observation name as keys, each containing a
-          List of observed 1D spectra
-
-        - Dict[str, List[WaveSpectrum2D]]
-          Dictionary with Spotter/Observation name as keys, each containing a
-          List of observed and reconstructed/computed 2D spectra
-
-        - List[List[WaveSpectrum2D]]]
-          List of partitioned data - each partition in itself is a list of
-          2D wavespectra ordered in time.
-
-        - Dict[str, List[List[WaveSpectrum2D]]]
-          Dictionary with Spotter/Observation name as keys, each containing a list
-          of partitioned data - each partition in itself is a list of
-          2D wavespectra ordered in time.
+        - Data in the same form it was saved.
     """
 
     with gzip.open(filename, 'rb') as file:
@@ -206,34 +172,8 @@ def save(_input: _UNION, filename: str, format='json', separate_spotters_into_fi
     compressed with gzip.
 
     :param _input:
-        - WaveSpectrum1D
-          A single 1D spectrum
-
-        - WaveSpectrum2D
-          A single 2D spectrum
-
-        - List[WaveSpectrum1D]
-          List of 1D spectra
-
-        - List[WaveSpectrum2D],
-          List of 2D spectra.
-
-        - Dict[str, List[WaveSpectrum1D]]
-          Dictionary with Spotter/Observation name as keys, each containing a
-          List of observed 1D spectra
-
-        - Dict[str, List[WaveSpectrum2D]]
-          Dictionary with Spotter/Observation name as keys, each containing a
-          List of observed and reconstructed/computed 2D spectra
-
-        - List[List[WaveSpectrum2D]]]
-          List of partitioned data - each partition in itself is a list of
-          2D wavespectra ordered in time.
-
-        - Dict[str, List[List[WaveSpectrum2D]]]
-          Dictionary with Spotter/Observation name as keys, each containing a list
-          of partitioned data - each partition in itself is a list of
-          2D wavespectra ordered in time.
+        - Data containing python primitives (dict/list) or any of the RogueWave
+          classes.
 
     :param filename: path to save data.
     :return: None
